@@ -8,8 +8,8 @@ by hand.
 import ast
 
 from parsimonious.exceptions import BadGrammar, UndefinedLabel
-from parsimonious.expressions import (Literal, Regex, Sequence, OneOf, AllOf,
-    Optional, ZeroOrMore, OneOrMore)
+from parsimonious.expressions import (Literal, Regex, Sequence, OneOf,
+    Lookahead, Optional, ZeroOrMore, OneOrMore, Not)
 from parsimonious.nodes import NodeVisitor
 from parsimonious.utils import StrAndRepr
 
@@ -77,7 +77,7 @@ class Grammar(StrAndRepr, dict):
         """
         tree = rule_grammar.parse(rules)
         if tree is None:
-            raise BadGrammar('There is an error in your rule definitions. '
+            raise BadGrammar('There is an error in your grammar definition. '
                              'Sorry for the vague error reporting at the '
                              'moment.')
 
@@ -128,6 +128,7 @@ class BootstrappingGrammar(Grammar):
         # Hard-code enough of the rules to parse the grammar that describes the
         # grammar description language, to bootstrap:
         ws = Regex(r'\s+', name='ws')
+        comment = Regex(r'#[^\r\n]*', name='comment')
         _ = Regex(r'[ \t]+', name='_')
         label = Regex(r'[a-zA-Z_][a-zA-Z_0-9]*', name='label')
         quantifier = Regex(r'[*+?]', name='quantifier')
@@ -141,19 +142,18 @@ class BootstrappingGrammar(Grammar):
         sequence = Sequence(term, OneOrMore(another_term), name='sequence')
         or_term = Sequence(_, Literal('/'), another_term, name='or_term')
         ored = Sequence(term, OneOrMore(or_term), name='ored')
-        and_term = Sequence(_, Literal('&'), another_term, name='and_term')
-        anded = Sequence(term, OneOrMore(and_term), name='anded')
-        poly_term = OneOf(anded, ored, sequence, name='poly_term')
-        rhs = OneOf(poly_term, term, name='rhs')
+        expression = OneOf(ored, sequence, term, name='expression')
         eol = Regex(r'[\r\n$]', name='eol')  # TODO: Support $.
-        rule = Sequence(Optional(ws), label, Optional(_), Literal('='),
-                        Optional(_), rhs, Optional(_), eol, name='rule')
-        rules = Sequence(OneOrMore(rule), Optional(ws), name='rules')
+        rule = Sequence(label, Optional(_), Literal('='), Optional(_),
+                        expression, Optional(_), Optional(comment), eol,
+                        name='rule')
+        rule_or_rubbish = OneOf(rule, ws, comment, name='rule_or_rubbish')
+        rules = OneOrMore(rule_or_rubbish, name='rules')
 
-        # Use those hard-coded rules to parse the (possibly more extensive)
-        # rule syntax. (For example, unless I start using parentheses in the
-        # rule language definition itself, I should never have to hard-code
-        # expressions for those above.)
+        # Use those hard-coded rules to parse the (more extensive) rule syntax.
+        # (For example, unless I start using parentheses in the rule language
+        # definition itself, I should never have to hard-code expressions for
+        # those above.)
         rule_tree = rules.parse(rule_syntax)
 
         # Turn the parse tree into a map of expressions:
@@ -161,36 +161,32 @@ class BootstrappingGrammar(Grammar):
 
 
 # The grammar for parsing PEG grammar definitions:
-# TODO: Support Not. Figure out how tightly it should bind.
-# TODO: Support comments.
-# This is a nice, simple grammar. We may someday add parentheses or support for
-# multi-line rules, but it's a safe bet that the future will always be a
+# This is a nice, simple grammar. We may someday add support for multi-line
+# rules or other sugar, but it's a safe bet that the future will always be a
 # superset of this.
 rule_syntax = (r'''
-    rules = rule+ ws?
-    rule = ws? label _? "=" _? rhs _? eol
+    rules = rule_or_rubbish+
+    rule_or_rubbish = rule / ws / comment
+    rule = label _? "=" _? expression _? comment? eol
     literal = ~"u?r?\"[^\"\\\\]*(?:\\\\.[^\"\\\\]*)*\""is
     eol = ~r"(?:[\r\n]|$)"
-    rhs = poly_term / term
-    poly_term = anded / ored / sequence
-    anded = term and_term+
-    and_term = _ "&" another_term
+    expression = ored / sequence / term
     or_term = _ "/" another_term
     ored = term or_term+
     sequence = term another_term+
     another_term = _ term
-    not_term = "!" term'''  # TODO: Half thought out. Make this work.
-    r'''
-    term = quantified / atom
+    not_term = "!" term
+    lookahead_term = "&" term
+    term = not_term / lookahead_term / quantified / atom
     quantified = atom quantifier
-    atom = label / literal / regex
+    atom = label / literal / regex / parenthesized
     regex = "~" literal ~"[ilmsux]*"i
-
+    parenthesized = "(" expression ")"
     quantifier = ~"[*+?]"
     label = ~"[a-zA-Z_][a-zA-Z_0-9]*"
-    _ = ~r"[ \t]+"'''  # horizontal whitespace
-    r'''
+    _ = ~r"[ \t]+"  # horizontal whitespace
     ws = ~r"\s+"
+    comment = ~r"#[^\r\n]*"
     ''')
 
 
@@ -209,16 +205,40 @@ class RuleVisitor(NodeVisitor):
     """
     quantifier_classes = {'?': Optional, '*': ZeroOrMore, '+': OneOrMore}
 
-    visit_rhs = visit_poly_term = visit_term = visit_atom = NodeVisitor.lift_child
+    visit_rule_or_rubbish = visit_expression = visit_term = visit_atom = \
+        NodeVisitor.lift_child
+
+    def visit_parenthesized(self, parenthesized, (left_paren, expression,
+                                                  right_paren)):
+        """Treat a parenthesized subexpression as just its contents.
+
+        Its position in the tree suffices to maintain its grouping semantics.
+
+        """
+        return expression
 
     def visit_quantified(self, quantified, (atom, quantifier)):
         return self.quantifier_classes[quantifier.text](atom)
 
-    def visit_rule(self, rule, (ws, label, _2, equals, _3, rhs, _4, eol)):
+    def visit_lookahead_term(self, lookahead_term, (ampersand, term)):
+        return Lookahead(term)
+
+    def visit_not_term(self, not_term, (exclamation, term)):
+        return Not(term)
+
+    def visit_ws(self, ws, visited_children):
+        """Stomp out ``ws`` nodes so visit_rules can easily filter them out."""
+
+    def visit_comment(self, comment, visited_children):
+        """Stomp out ``comment`` nodes so visit_rules can easily filter them
+        out."""
+
+    def visit_rule(self, rule, (label, _2, equals, _3, expression, _4, comment,
+                                eol)):
         """Assign a name to the Expression and return it."""
         label = unicode(label)  # Turn lazy reference back into text.  # TODO: Remove backtracking.
-        rhs.name = label  # Assign a name to the expr.
-        return rhs
+        expression.name = label  # Assign a name to the expr.
+        return expression
 
     def visit_sequence(self, sequence, (term, other_terms)):
         """A parsed Sequence looks like [term node, OneOrMore node of
@@ -236,17 +256,6 @@ class RuleVisitor(NodeVisitor):
         """
         return term
 
-    def visit_anded(self, anded, (first_term, other_terms)):
-        return AllOf(first_term, *other_terms)
-
-    def visit_and_term(self, and_term, (_, ampersand, term)):
-        """Return just the term from an ``and_term``.
-
-        We already know it's going to be anded, from the containing ``anded``.
-
-        """
-        return term
-
     def visit_another_term(self, another_term, (_, term)):
         """Strip off the space, and just return the actual term involved in
         ``another_term``.
@@ -294,14 +303,14 @@ class RuleVisitor(NodeVisitor):
         regex rule. Most of these kept-around nodes are subsequently thrown
         away by the other visitor methods.
 
-        We can't simple hang the visited children off the original node; that
+        We can't simply hang the visited children off the original node; that
         would be disastrous if the node occurred in more than one place in the
         tree.
 
         """
         return visited_children or node  # should semantically be a tuple
 
-    def visit_rules(self, node, (rules, ws)):
+    def visit_rules(self, node, rule_or_rubbishes):
         """Collate all the rules into a map. Return (map, default rule).
 
         The default rule is the first one. Or, if you have more than one rule
@@ -309,9 +318,11 @@ class RuleVisitor(NodeVisitor):
         override the default rule when you extend a grammar.)
 
         """
-        # TODO: Too big. Break up.
         rule_map = {}
 
+        # Drop the ws and comments:
+        rules = [r for r in rule_or_rubbishes if r is not None]
+
         def resolve_refs(expr):
             """Turn references into the things they actually reference.
 
@@ -353,7 +364,7 @@ rule_grammar = BootstrappingGrammar(rule_syntax)
 rule_grammar = Grammar(rule_syntax)
 
 # TODO: Teach Expression trees how to spit out Python representations of
-# themselves. Then we can just paste that in about, and we won't have to
+# themselves. Then we can just paste that in above, and we won't have to
 # bootstrap on import. Though it'll be a little less DRY. [Ah, but this is not
 # so clean, because it would have to output multiple statements to get multiple
 # refs to a single expression hooked up.]
