@@ -15,8 +15,7 @@ def test_lists_vs_dicts():
     dict_time = timeit('item = d[9000]', 'd = dict((x, 0) for x in xrange(10000))')
 
     # Dicts take about 1.6x as long as lists in Python 2.6 and 2.7.
-    print '%s < %s' % (list_time, dict_time)
-    ok_(list_time < dict_time)
+    ok_(list_time < dict_time, '%s < %s' % (list_time, dict_time))
 
 
 def test_call_vs_inline():
@@ -28,8 +27,7 @@ def test_call_vs_inline():
 
     # Calling a function is pretty fast; it takes just 1.2x as long as the
     # global var access and addition in l[0] += 1.
-    print '%s (no call) < %s (call)' % (no_call, call)
-    ok_(no_call < call)
+    ok_(no_call < call, '%s (no call) < %s (call)' % (no_call, call))
 
 
 def test_startswith_vs_regex():
@@ -43,5 +41,5 @@ def test_startswith_vs_regex():
                              "t = 'this is the finest hello ever'")
 
     # Regexes take 2.24x as long as simple string matching.
-    print '%s (startswith) < %s (re)' % (startswith_time, re_time)
-    ok_(startswith_time < re_time)
+    ok_(startswith_time < re_time,
+        '%s (startswith) < %s (re)' % (startswith_time, re_time))
