@@ -59,14 +59,15 @@ class BootstrapingGrammarTests(TestCase):
 
         ok_(rule_grammar['ored'].parse('this / that+ / "other"'))
 
-        ok_(rule_grammar['anded'].parse('this & that+ & "other"'))
+        # + is higher precedence than &, so 'anded' should match the whole
+        # thing:
+        ok_(rule_grammar['lookahead_term'].parse('&this+'))
 
-        ok_(rule_grammar['poly_term'].parse('this & that+ & "other"'))
-        ok_(rule_grammar['poly_term'].parse('this / that? / "other"+'))
-        ok_(rule_grammar['poly_term'].parse('this? that other*'))
-
-        ok_(rule_grammar['rhs'].parse('this'))
-        ok_(rule_grammar['rhs'].parse('this? that other*'))
+        ok_(rule_grammar['expression'].parse('this'))
+        ok_(rule_grammar['expression'].parse('this? that other*'))
+        ok_(rule_grammar['expression'].parse('&this / that+ / "other"'))
+        ok_(rule_grammar['expression'].parse('this / that? / "other"+'))
+        ok_(rule_grammar['expression'].parse('this? that other*'))
 
         ok_(rule_grammar['rule'].parse('this = that\r'))
         ok_(rule_grammar['rule'].parse('this = the? that other* \t\r'))
@@ -170,3 +171,52 @@ class GrammarTests(TestCase):
     def test_bad_grammar(self):
         """Constructing a Grammar with bad rules should raise BadGrammar."""
         assert_raises(BadGrammar, Grammar, 'just a bunch of junk')
+
+    def test_comments(self):
+        """Test tolerance of comments and blank lines in and around rules."""
+        grammar = Grammar(r"""# This is a grammar.
+
+                          # It sure is.
+                          bold_text  = stars text stars  # nice
+                          text       = ~"[A-Z 0-9]*"i #dude
+                          
+                          
+                          stars      = "**"
+                          # Pretty good
+                          #Oh yeah.#""")  # Make sure a comment doesn't need a
+                                          # \n or \r to end.
+        eq_(str(grammar), '''bold_text = stars text stars\n'''
+                          '''text = ~"[A-Z 0-9]*"i\n'''
+                          '''stars = "**"''')
+
+    def test_not(self):
+        """Make sure "not" predicates get parsed and work properly."""
+        grammar = Grammar(r'''not_arp = !"arp" ~"[a-z]+"''')
+        eq_(grammar.parse('arp'), None)
+        ok_(grammar.parse('argle') is not None)
+
+    def test_lookahead(self):
+        grammar = Grammar(r'''starts_with_a = &"a" ~"[a-z]+"''')
+        eq_(grammar.parse('burp'), None)
+
+        s = 'arp'
+        eq_(grammar.parse('arp'), Node('starts_with_a', s, 0, 3, children=[
+                                      Node('', s, 0, 0),
+                                      Node('', s, 0, 3)]))
+
+    def test_parens(self):
+        grammar = Grammar(r'''sequence = "chitty" (" " "bang")+''')
+        # Make sure it's not as if the parens aren't there:
+        eq_(grammar.parse('chitty bangbang'), None)
+
+        s = 'chitty bang bang'
+        eq_(str(grammar.parse(s)),
+            """<Node called "sequence" matching "chitty bang bang">
+    <Node matching "chitty">
+    <Node matching " bang bang">
+        <Node matching " bang">
+            <Node matching " ">
+            <Node matching "bang">
+        <Node matching " bang">
+            <Node matching " ">
+            <Node matching "bang">""")
